==> ebclient-1.0.tar.gz: ebclient-1.0/ebclient/eb_request.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from ebclient.eb_utils import EBUtils
from ebclient.eb_consts import EBConsts
from ebclient.eb_configuration import SimpleRetry
from ebclient.crypto_util import *
from ebclient.errors import *
from ebclient.uo import *
import requests

__author__ = 'Enigma Bridge Ltd'


logger = logging.getLogger(__name__)


class RequestHolder(object):
    """
    Class holding the general request
    """
    def __init__(self, body=None, api_object=None, nonce=None, endpoint=None, config=None, url=None, headers=None,
                 *args, **kwargs):
        self.body = body
        self.api_object = api_object
        self.api_method = 'ProcessData'
        self.nonce = nonce
        self.endpoint = endpoint
        self.configuration = config
        self.url = url
        self.headers = headers

    def __repr__(self):
        return '%s(body=%r, api_object=%r, api_method=%r, nonce=%r, endpoint=%r, config=%r, ' \
               'url=%r, headers=%r' \
               % (self.__class__, self.body, self.api_object, self.api_method, self.nonce, self.endpoint,
                  self.configuration, self.url, self.headers)


class ResponseHolder(object):
    """
    Class holding the general response
    """
    def __init__(self, response=None, status=0x0, *args, **kwargs):
        self.response = response
        self.status = status

    def __repr__(self):
        return '%s(response=%r, status=%r)' % (self.__class__, self.response, self.status)


class RequestCall(object):
    """
    Class responsible for making a request on the EB API according to the configuration.
    """

    def __init__(self, request=None, response_checker=None, *args, **kwargs):
        self.request = request
        self.response = None
        self.last_resp = None
        self.response_checker = response_checker
        pass

    def call(self, request=None, *args, **kwargs):
        """
        Calls multiple time - with retry.

        :param request:
        :return: response
        """
        if request is not None:
            self.request = request

        retry = self.request.configuration.retry
        if not isinstance(retry, SimpleRetry):
            raise Error('Currently only the fast retry strategy is supported')

        last_exception = None
        for i in range(0, retry.max_retry):
            try:
                if i > 0:
                    retry.sleep_jitter()

                self.call_once()
                return self.response

            except Exception as ex:
                last_exception = RequestFailed(message='Request failed', cause=ex)
                logger.debug("Request %d failed, exception: %s" % (i, ex))

                # Last exception - throw it here to have a stack
                if i+1 == retry.max_retry:
                    raise last_exception

        raise last_exception

    @staticmethod
    def field_to_long(value):
        """
        Converts given value to long if possible, otherwise None is returned.

        :param value:
        :return:
        """
        if isinstance(value, (types.LongType, types.IntType)):
            return long(value)
        elif isinstance(value, basestring):
            return bytes_to_long(from_hex(value))
        else:
            return None

    def build_url(self):
        """
        Construct URL
        e.g.,: http://site2.enigmabridge.com:12000/1.0/testAPI/GetAllAPIKeys/abcdef012345

        :return:
        """
        url = "%s/1.0/%s/%s/%s" % (
            self.request.endpoint.get_url(),
            self.request.api_object,
            self.request.api_method,
            to_hex(self.request.nonce)
        )
        return url

    @staticmethod
    def get_text_status(json):
        """
        Extracts textual status from the response - statusdetail, if present.
        Otherwise extracts status field.

        :param json:
        :return:
        """
        if json is None:
            return None
        elif 'statusdetail' in json:
            return json['statusdetail']
        elif 'status' in json:
            return json['status']
        else:
            return None

    def call_once(self, request=None, *args, **kwargs):
        """
        Performs one API request.
        Raises exception on failure.

        :param request:
        :param args:
        :param kwargs:
        :return: response
        """
        if request is not None:
            self.request = request

        config = self.request.configuration
        if config.http_method != EBConsts.HTTP_METHOD_POST or config.method != EBConsts.METHOD_REST:
            raise Error('Not implemented yet, only REST POST method is allowed')

        url = self.request.url if self.request.url is not None else self.build_url()
        logger.debug("URL to call: %s", url)

        # Do the request
        resp = requests.post(url, json=self.request.body, timeout=config.timeout, headers=self.request.headers)
        self.last_resp = resp
        return self.check_response(resp)

    def check_response(self, resp):
        """
        Checks response after request was made.
        Checks status of the response, mainly

        :param resp:
        :return:
        """

        # For successful API call, response code will be 200 (OK)
        if resp.ok:
            json = resp.json()
            self.response = ResponseHolder()
            self.response.response = json

            # Check the code
            if 'status' not in json:
                raise InvalidResponse('No status field')

            self.response.status = self.field_to_long(json['status'])
            if self.response.status != EBConsts.STATUS_OK:
                txt_status = self.get_text_status(json)
                raise InvalidStatus('Status is %s (%04X)'
                                    % (txt_status if txt_status is not None else "", self.response.status))

            if self.response_checker is not None:
                self.response_checker(self.response)

            return self.response

        else:
            # If response code is not ok (200), print the resulting http error code with description
            resp.raise_for_status()
        pass

    def __repr__(self):
        return '%s(request=%r, response_checker=%r, response=%r, last_resp=%r)' \
               % (self.__class__, self.request, self.response_checker, self.response, self.last_resp)

==> ebclient-1.1.16.tar.gz: ebclient-1.1.16/ebclient/eb_request.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
from ebclient.eb_utils import EBUtils
from ebclient.eb_consts import EBConsts
from ebclient.eb_configuration import SimpleRetry
from ebclient.crypto_util import *
from ebclient.errors import *
from ebclient.uo import *
import requests
if sys.version_info[0] > 2:
    from past.builtins import basestring    # pip install future
    from past.builtins import long

__author__ = 'Enigma Bridge Ltd'


logger = logging.getLogger(__name__)


class RequestHolder(object):
    """
    Class holding the general request
    """
    def __init__(self, body=None, api_object=None, nonce=None, endpoint=None, config=None, url=None, headers=None,
                 *args, **kwargs):
        self.body = body
        self.api_object = api_object
        self.api_method = 'ProcessData'
        self.nonce = nonce
        self.endpoint = endpoint
        self.configuration = config
        self.url = url
        self.headers = headers

    def __repr__(self):
        return '%s(body=%r, api_object=%r, api_method=%r, nonce=%r, endpoint=%r, config=%r, ' \
               'url=%r, headers=%r' \
               % (self.__class__, self.body, self.api_object, self.api_method, self.nonce, self.endpoint,
                  self.configuration, self.url, self.headers)


class ResponseHolder(object):
    """
    Class holding the general response
    """
    def __init__(self, response=None, status=0x0, *args, **kwargs):
        self.response = response
        self.status = status

    def __repr__(self):
        return '%s(response=%r, status=%r)' % (self.__class__, self.response, self.status)


class RequestCall(object):
    """
    Class responsible for making a request on the EB API according to the configuration.
    """

    def __init__(self, request=None, response_checker=None, *args, **kwargs):
        self.request = request
        self.response = None
        self.last_resp = None
        self.response_checker = response_checker
        pass

    def call(self, request=None, *args, **kwargs):
        """
        Calls multiple time - with retry.

        :param request:
        :return: response
        """
        if request is not None:
            self.request = request

        retry = self.request.configuration.retry
        if not isinstance(retry, SimpleRetry):
            raise Error('Currently only the fast retry strategy is supported')

        last_exception = None
        for i in range(0, retry.max_retry):
            try:
                if i > 0:
                    retry.sleep_jitter()

                self.call_once()
                return self.response

            except Exception as ex:
                last_exception = RequestFailed(message='Request failed', cause=ex)
                logger.debug("Request %d failed, exception: %s" % (i, ex))

                # Last exception - throw it here to have a stack
                if i+1 == retry.max_retry:
                    raise last_exception

        raise last_exception

    @staticmethod
    def field_to_long(value):
        """
        Converts given value to long if possible, otherwise None is returned.

        :param value:
        :return:
        """
        if isinstance(value, (int, long)):
            return long(value)
        elif isinstance(value, basestring):
            return bytes_to_long(from_hex(value))
        else:
            return None

    def build_url(self):
        """
        Construct URL
        e.g.,: http://site2.enigmabridge.com:12000/1.0/testAPI/GetAllAPIKeys/abcdef012345

        :return:
        """
        url = "%s/1.0/%s/%s/%s" % (
            self.request.endpoint.get_url(),
            self.request.api_object,
            self.request.api_method,
            to_hex(self.request.nonce)
        )
        return url

    @staticmethod
    def get_text_status(json):
        """
        Extracts textual status from the response - statusdetail, if present.
        Otherwise extracts status field.

        :param json:
        :return:
        """
        if json is None:
            return None
        elif 'statusdetail' in json:
            return json['statusdetail']
        elif 'status' in json:
            return json['status']
        else:
            return None

    def call_once(self, request=None, *args, **kwargs):
        """
        Performs one API request.
        Raises exception on failure.

        :param request:
        :param args:
        :param kwargs:
        :return: response
        """
        if request is not None:
            self.request = request

        config = self.request.configuration
        if config.http_method != EBConsts.HTTP_METHOD_POST or config.method != EBConsts.METHOD_REST:
            raise Error('Not implemented yet, only REST POST method is allowed')

        url = self.request.url if self.request.url is not None else self.build_url()
        logger.debug("URL to call: %s", url)

        # Do the request
        resp = requests.post(url, json=self.request.body, timeout=config.timeout, headers=self.request.headers)
        self.last_resp = resp
        return self.check_response(resp)

    def check_response(self, resp):
        """
        Checks response after request was made.
        Checks status of the response, mainly

        :param resp:
        :return:
        """

        # For successful API call, response code will be 200 (OK)
        if resp.ok:
            json = resp.json()
            self.response = ResponseHolder()
            self.response.response = json

            # Check the code
            if 'status' not in json:
                raise InvalidResponse('No status field')

            self.response.status = self.field_to_long(json['status'])
            if self.response.status != EBConsts.STATUS_OK:
                txt_status = self.get_text_status(json)
                raise InvalidStatus('Status is %s (%04X)'
                                    % (txt_status if txt_status is not None else "", self.response.status))

            if self.response_checker is not None:
                self.response_checker(self.response)

            return self.response

        else:
            # If response code is not ok (200), print the resulting http error code with description
            resp.raise_for_status()
        pass

    def __repr__(self):
        return '%s(request=%r, response_checker=%r, response=%r, last_resp=%r)' \
               % (self.__class__, self.request, self.response_checker, self.response, self.last_resp)
